package com.sun.org.apache.bcel.internal.classfile;

import java.util.HashMap;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;
import java.util.Map;

public final class Unknown extends Attribute
{
    private byte[] bytes;
    private final String name;
    private static final Map<String, Unknown> unknown_attributes;
    
    static Unknown[] getUnknownAttributes() {
        Unknown[] unknowns = new Unknown[Unknown.unknown_attributes.size()];
        Unknown.unknown_attributes.values().toArray(unknowns);
        Unknown.unknown_attributes.clear();
        return unknowns;
    }
    
    public Unknown(Unknown c) {
        this(c.getNameIndex(), c.getLength(), c.getBytes(), c.getConstantPool());
    }
    
    public Unknown(int name_index, int length, byte[] bytes, ConstantPool constant_pool) {
        super((byte)(-1), name_index, length, constant_pool);
        this.bytes = bytes;
        this.name = ((ConstantUtf8)constant_pool.getConstant(name_index, (byte)1)).getBytes();
        Unknown.unknown_attributes.put(this.name, this);
    }
    
    Unknown(int name_index, int length, DataInput input, ConstantPool constant_pool) throws IOException {
        this(name_index, length, (byte[])null, constant_pool);
        if (length > 0) {
            input.readFully(this.bytes = new byte[length]);
        }
    }
    
    @Override
    public void accept(Visitor v) {
        v.visitUnknown(this);
    }
    
    @Override
    public final void dump(DataOutputStream file) throws IOException {
        super.dump(file);
        if (super.getLength() > 0) {
            file.write(this.bytes, 0, super.getLength());
        }
    }
    
    public final byte[] getBytes() {
        return this.bytes;
    }
    
    @Override
    public final String getName() {
        return this.name;
    }
    
    public final void setBytes(byte[] bytes) {
        this.bytes = bytes;
    }
    
    @Override
    public final String toString() {
        if (super.getLength() == 0 || this.bytes == null) {
            return "(Unknown attribute " + this.name + ")";
        }
        String hex;
        if (super.getLength() > 10) {
            byte[] tmp = new byte[10];
            System.arraycopy(this.bytes, 0, tmp, 0, 10);
            hex = Utility.toHexString(tmp) + "... (truncated)";
        }
        else {
            hex = Utility.toHexString(this.bytes);
        }
        return "(Unknown attribute " + this.name + ": " + hex + ")";
    }
    
    @Override
    public Attribute copy(ConstantPool _constant_pool) {
        Unknown c = (Unknown)this.clone();
        if (this.bytes != null) {
            c.bytes = new byte[this.bytes.length];
            System.arraycopy(this.bytes, 0, c.bytes, 0, this.bytes.length);
        }
        c.setConstantPool(_constant_pool);
        return c;
    }
    
    static {
        unknown_attributes = new HashMap();
    }
}
